@@ -1,6 +1,6 @@
-# Copyright 1999-2009 Gentoo Foundation
+#!/bin/bash
+# Copyright 1999-2011 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Id: isolated-functions.sh 15146 2009-12-24 05:11:32Z zmedico $
 
 # We need this next line for "die" and "assert". It expands
 # It _must_ preceed all the calls to die and assert.
@@ -15,6 +15,32 @@ assert() {
 	done
 }
 
+assert_sigpipe_ok() {
+	# When extracting a tar file like this:
+	#
+	#     bzip2 -dc foo.tar.bz2 | tar xof -
+	#
+	# For some tar files (see bug #309001), tar will
+	# close its stdin pipe when the decompressor still has
+	# remaining data to be written to its stdout pipe. This
+	# causes the decompressor to be killed by SIGPIPE. In
+	# this case, we want to ignore pipe writers killed by
+	# SIGPIPE, and trust the exit status of tar. We refer
+	# to the bash manual section "3.7.5 Exit Status"
+	# which says, "When a command terminates on a fatal
+	# signal whose number is N, Bash uses the value 128+N
+	# as the exit status."
+
+	local x pipestatus=${PIPESTATUS[*]}
+	for x in $pipestatus ; do
+		# Allow SIGPIPE through (128 + 13)
+		[[ $x -ne 0 && $x -ne ${PORTAGE_SIGPIPE_STATUS:-141} ]] && die "$@"
+	done
+
+	# Require normal success for the last process (tar).
+	[[ $x -eq 0 ]] || die "$@"
+}
+
 shopt -s extdebug
 
 # dump_trace([number of funcs on stack to skip],
@@ -43,7 +69,7 @@ dump_trace() {
 	eerror "Call stack:"
 	while (( n > ${strip} )) ; do
 		funcname=${FUNCNAME[${n} - 1]}
-		sourcefile=$(basename ${BASH_SOURCE[${n}]})
+		sourcefile=$(basename "${BASH_SOURCE[${n}]}")
 		lineno=${BASH_LINENO[${n} - 1]}
 		# Display function arguments
 		args=
@@ -70,6 +96,17 @@ nonfatal() {
 	PORTAGE_NONFATAL=1 "$@"
 }
 
+helpers_die() {
+	case "${EAPI:-0}" in
+		0|1|2|3)
+			echo -e "$@" >&2
+			;;
+		*)
+			die "$@"
+			;;
+	esac
+}
+
 die() {
 	if [[ $PORTAGE_NONFATAL -eq 1 ]]; then
 		echo -e " $WARN*$NORMAL ${FUNCNAME[1]}: WARNING: $@" >&2
@@ -99,9 +136,17 @@ die() {
 		(( n-- ))
 	done
 
-	eerror "ERROR: $CATEGORY/$PF failed:"
+	# When a helper binary dies automatically in EAPI 4 and later, we don't
+	# get a stack trace, so at least report the phase that failed.
+	local phase_str=
+	[[ -n $EBUILD_PHASE ]] && phase_str=" ($EBUILD_PHASE phase)"
+	eerror "ERROR: $CATEGORY/$PF failed${phase_str}:"
 	eerror "  ${*:-(no error message)}"
 	eerror
+	# dump_trace is useless when the main script is a helper binary
+	local main_index
+	(( main_index = ${#BASH_SOURCE[@]} - 1 ))
+	if has ${BASH_SOURCE[$main_index]##*/} ebuild.sh misc-functions.sh ; then
 	dump_trace 2 ${filespacing} ${linespacing}
 	eerror "  $(printf "%${filespacing}s" "${BASH_SOURCE[1]##*/}"), line $(printf "%${linespacing}s" "${BASH_LINENO[0]}"):  Called die"
 	eerror "The specific snippet of code:"
@@ -123,10 +168,11 @@ die() {
 		    # erase the line and set the hold buffer to it (thus
 		    # erasing the hold buffer in the process)
 		    /[^\]$/{s:^.*$::;h}' \
-		${BASH_SOURCE[1]} \
+		"${BASH_SOURCE[1]}" \
 		| sed -e '1d' -e 's:^:RETAIN-LEADING-SPACE:' \
 		| while read -r n ; do eerror "  ${n#RETAIN-LEADING-SPACE}" ; done
 	eerror
+	fi
 	eerror "If you need support, post the output of 'emerge --info =$CATEGORY/$PF',"
 	eerror "the complete build log and the output of 'emerge -pqv =$CATEGORY/$PF'."
 	if [[ -n ${EBUILD_OVERLAY_ECLASSES} ]] ; then
@@ -178,7 +224,8 @@ die() {
 	fi
 	eerror "S: '${S}'"
 
-	[ -n "$EBUILD_EXIT_STATUS_FILE" ] && > "$EBUILD_EXIT_STATUS_FILE"
+	[[ -n $PORTAGE_EBUILD_EXIT_FILE ]] && > "$PORTAGE_EBUILD_EXIT_FILE"
+	[[ -n $PORTAGE_IPC_DAEMON ]] && "$PORTAGE_BIN_PATH"/ebuild-ipc exit 1
 
 	# subshell die support
 	[[ $BASHPID = $EBUILD_MASTER_PID ]] || kill -s SIGTERM $EBUILD_MASTER_PID
@@ -326,10 +373,10 @@ _eend() {
 	fi
 
 	if [[ ${RC_ENDCOL} == "yes" ]] ; then
-		echo -e "${ENDCOL}  ${msg}"
+		echo -e "${ENDCOL} ${msg}"
 	else
 		[[ ${LAST_E_CMD} == ebegin ]] || LAST_E_LEN=0
-		printf "%$(( COLS - LAST_E_LEN - 6 ))s%b\n" '' "${msg}"
+		printf "%$(( COLS - LAST_E_LEN - 7 ))s%b\n" '' "${msg}"
 	fi
 
 	return ${retval}
@@ -397,7 +444,7 @@ get_KV() {
 }
 
 unset_colors() {
-	COLS="25 80"
+	COLS=80
 	ENDCOL=
 
 	GOOD=
@@ -412,13 +459,10 @@ set_colors() {
 	COLS=${COLUMNS:-0}      # bash's internal COLUMNS variable
 	(( COLS == 0 )) && COLS=$(set -- $(stty size 2>/dev/null) ; echo $2)
 	(( COLS > 0 )) || (( COLS = 80 ))
-	COLS=$((${COLS} - 8))	# width of [ ok ] == 7
-	# Adjust COLS so that eend works properly on a standard BSD console.
-	[[ $TERM = cons25 || $TERM = dumb ]] && ((COLS--))
 
 	# Now, ${ENDCOL} will move us to the end of the
 	# column;  irregardless of character width
-	ENDCOL=$'\e[A\e['${COLS}'C'
+	ENDCOL=$'\e[A\e['$(( COLS - 8 ))'C'
 	if [ -n "${PORTAGE_COLORMAP}" ] ; then
 		eval ${PORTAGE_COLORMAP}
 	else
@@ -504,10 +548,10 @@ save_ebuild_env() {
 
 		# misc variables inherited from the calling environment
 		unset COLORTERM DISPLAY EDITOR LESS LESSOPEN LOGNAME LS_COLORS PAGER \
-			TERM TERMCAP USER
+			TERM TERMCAP USER ftp_proxy http_proxy no_proxy
 
 		# other variables inherited from the calling environment
-		unset ECHANGELOG_USER GPG_AGENT_INFO \
+		unset CVS_RSH ECHANGELOG_USER GPG_AGENT_INFO \
 		SSH_AGENT_PID SSH_AUTH_SOCK STY WINDOW XAUTHORITY
 
 		# CCACHE and DISTCC config
@@ -523,7 +567,7 @@ save_ebuild_env() {
 		done
 		unset x
 
-		unset -f assert dump_trace die diefunc \
+		unset -f assert assert_sigpipe_ok dump_trace die diefunc \
 			quiet_mode vecho elog_base eqawarn elog \
 			esyslog einfo einfon ewarn eerror ebegin _eend eend KV_major \
 			KV_minor KV_micro KV_to_int get_KV unset_colors set_colors has \
@@ -531,16 +575,15 @@ save_ebuild_env() {
 			hasg hasgq hasv hasq qa_source qa_call \
 			addread addwrite adddeny addpredict _sb_append_var \
 			lchown lchgrp esyslog use usev useq has_version portageq \
-			best_version use_with use_enable register_die_hook check_KV \
+			best_version use_with use_enable register_die_hook \
 			keepdir unpack strip_duplicate_slashes econf einstall \
 			dyn_setup dyn_unpack dyn_clean into insinto exeinto docinto \
-			insopts diropts exeopts libopts \
+			insopts diropts exeopts libopts docompress \
 			abort_handler abort_prepare abort_configure abort_compile \
 			abort_test abort_install dyn_prepare dyn_configure \
 			dyn_compile dyn_test dyn_install \
 			dyn_preinst dyn_help debug-print debug-print-function \
-			debug-print-section inherit EXPORT_FUNCTIONS newdepend newrdepend \
-			newpdepend do_newdepend remove_path_entry \
+			debug-print-section inherit EXPORT_FUNCTIONS remove_path_entry \
 			save_ebuild_env filter_readonly_variables preprocess_ebuild_env \
 			source_all_bashrcs ebuild_main \
 			ebuild_phase ebuild_phase_with_hooks \
@@ -551,24 +594,16 @@ save_ebuild_env() {
 		# portage config variables and variables set directly by portage
 		unset ACCEPT_LICENSE BAD BRACKET BUILD_PREFIX COLS \
 			DISTCC_DIR DISTDIR DOC_SYMLINKS_DIR \
-			EBUILD_EXIT_STATUS_FILE EBUILD_FORCE_TEST EBUILD_MASTER_PID \
-			ECLASSDIR ECLASS_DEPTH ENDCOL FAKEROOTKEY \
+			EBUILD_FORCE_TEST EBUILD_MASTER_PID \
+			ECLASS_DEPTH ENDCOL FAKEROOTKEY \
 			GOOD HILITE HOME \
 			LAST_E_CMD LAST_E_LEN LD_PRELOAD MISC_FUNCTIONS_ARGS MOPREFIX \
 			NOCOLOR NORMAL PKGDIR PKGUSE PKG_LOGDIR PKG_TMPDIR \
-			PORTAGE_ACTUAL_DISTDIR PORTAGE_ARCHLIST \
-			PORTAGE_BASHRC PORTAGE_BASHRCS_SOURCED \
-			PORTAGE_BINPKG_TAR_OPTS PORTAGE_BINPKG_TMPFILE PORTAGE_BUILDDIR \
-			PORTAGE_COLORMAP PORTAGE_CONFIGROOT PORTAGE_DEBUG \
-			PORTAGE_DEPCACHEDIR PORTAGE_GID PORTAGE_INST_GID \
-			PORTAGE_INST_UID PORTAGE_LOG_FILE PORTAGE_MASTER_PID \
-			PORTAGE_NONFATAL PORTAGE_QUIET \
-			PORTAGE_REPO_NAME PORTAGE_RESTRICT PORTAGE_UPDATE_ENV \
-			PORTAGE_VERBOSE PORTAGE_WORKDIR_MODE PORTDIR \
-			PORTDIR_OVERLAY ${!PORTAGE_SANDBOX_*} PREROOTPATH \
-			PROFILE_PATHS PWORKDIR QA_INTERCEPTORS \
-			RC_DEFAULT_INDENT RC_DOT_PATTERN RC_ENDCOL \
-			RC_INDENTATION READONLY_EBUILD_METADATA READONLY_PORTAGE_VARS \
+			PORTAGE_BASHRCS_SOURCED PORTAGE_NONFATAL PORTAGE_QUIET \
+			PORTAGE_SANDBOX_DENY PORTAGE_SANDBOX_PREDICT \
+			PORTAGE_SANDBOX_READ PORTAGE_SANDBOX_WRITE PREROOTPATH \
+			QA_INTERCEPTORS \
+			RC_DEFAULT_INDENT RC_DOT_PATTERN RC_ENDCOL RC_INDENTATION  \
 			ROOT ROOTPATH RPMDIR TEMP TMP TMPDIR USE_EXPAND \
 			WARN XARGS _RC_GET_KV_CACHE
 
