@@ -1,6 +1,5 @@
-# Copyright 1999-2009 Gentoo Foundation
+# Copyright 1999-2011 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Id: BinpkgVerifier.py 15647 2010-03-02 20:56:10Z zmedico $
 
 from _emerge.AsynchronousTask import AsynchronousTask
 from portage.util import writemsg
@@ -8,12 +7,11 @@ import sys
 import portage
 from portage import os
 from portage import _encodings
-from portage import _unicode_encode
+from portage import _unicode_decode
 from portage.package.ebuild.fetch import _checksum_failure_temp_file
-import codecs
 
 class BinpkgVerifier(AsynchronousTask):
-	__slots__ = ("logfile", "pkg",)
+	__slots__ = ("logfile", "pkg", "scheduler")
 
 	def _start(self):
 		"""
@@ -28,33 +26,21 @@ class BinpkgVerifier(AsynchronousTask):
 		rval = os.EX_OK
 		stdout_orig = sys.stdout
 		stderr_orig = sys.stderr
-		log_file = None
-		if self.background and self.logfile is not None:
-			if sys.hexversion >= 0x3000000:
-				# Since we are replacing the sys.std* streams,
-				# we need to use the normal open() function
-				# so that we get the right class (otherwise our
-				# code that expects the 'buffer' attribute
-				# will break).
-				log_file = open(_unicode_encode(self.logfile,
-					encoding=_encodings['fs'], errors='strict'),
-					mode='a', encoding=_encodings['content'],
-					errors='backslashreplace')
-			else:
-				# For python2, sys.std* are expected to be binary streams.
-				log_file = open(_unicode_encode(self.logfile,
-					encoding=_encodings['fs'], errors='strict'),
-					mode='ab')
+		global_havecolor = portage.output.havecolor
+		out = portage.StringIO()
+		file_exists = True
 		try:
-			if log_file is not None:
-				sys.stdout = log_file
-				sys.stderr = log_file
+			sys.stdout = out
+			sys.stderr = out
+			if portage.output.havecolor:
+				portage.output.havecolor = not self.background
 			try:
 				bintree.digestCheck(pkg)
 			except portage.exception.FileNotFound:
 				writemsg("!!! Fetching Binary failed " + \
 					"for '%s'\n" % pkg.cpv, noiselevel=-1)
 				rval = 1
+				file_exists = False
 			except portage.exception.DigestException as e:
 				writemsg("\n!!! Digest verification failed:\n",
 					noiselevel=-1)
@@ -68,14 +54,8 @@ class BinpkgVerifier(AsynchronousTask):
 					noiselevel=-1)
 				rval = 1
 			if rval == os.EX_OK:
-				# If this was successful, discard the log here since otherwise
-				# we'll get multiple logs for the same package.
-				if log_file is not None:
-					try:
-						os.unlink(self.logfile)
-					except OSError:
-						pass
-			else:
+				pass
+			elif file_exists:
 				pkg_path = bintree.getname(pkg.cpv)
 				head, tail = os.path.split(pkg_path)
 				temp_filename = _checksum_failure_temp_file(head, tail)
@@ -84,8 +64,13 @@ class BinpkgVerifier(AsynchronousTask):
 		finally:
 			sys.stdout = stdout_orig
 			sys.stderr = stderr_orig
-			if log_file is not None:
-				log_file.close()
+			portage.output.havecolor = global_havecolor
+
+		msg = _unicode_decode(out.getvalue(),
+			encoding=_encodings['content'], errors='replace')
+		if msg:
+			self.scheduler.output(msg, log_path=self.logfile,
+				background=self.background)
 
 		self.returncode = rval
 		self.wait()
