@@ -1,16 +1,16 @@
-# Copyright 1999-2009 Gentoo Foundation
+# Copyright 1999-2011 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Id: SpawnProcess.py 14942 2009-12-06 11:00:34Z zmedico $
 
 from _emerge.SubProcess import SubProcess
-from _emerge.PollConstants import PollConstants
 import sys
 from portage.cache.mappings import slot_dict_class
 import portage
+from portage import _encodings
+from portage import _unicode_encode
 from portage import os
 import fcntl
 import errno
-import array
+import gzip
 
 class SpawnProcess(SubProcess):
 
@@ -72,13 +72,20 @@ class SpawnProcess(SubProcess):
 		else:
 			fd_pipes[0] = fd_pipes_orig[0]
 
-		files.process = os.fdopen(master_fd, 'rb')
+		# WARNING: It is very important to use unbuffered mode here,
+		# in order to avoid issue 5380 with python3.
+		files.process = os.fdopen(master_fd, 'rb', 0)
 		if logfile is not None:
 
 			fd_pipes[1] = slave_fd
 			fd_pipes[2] = slave_fd
 
-			files.log = open(logfile, mode='ab')
+			files.log = open(_unicode_encode(logfile,
+				encoding=_encodings['fs'], errors='strict'), mode='ab')
+			if logfile.endswith('.gz'):
+				files.log = gzip.GzipFile(filename='', mode='ab',
+					fileobj=files.log)
+
 			portage.util.apply_secpass_permissions(logfile,
 				uid=portage.portage_uid, gid=portage.portage_gid,
 				mode=0o660)
@@ -121,7 +128,7 @@ class SpawnProcess(SubProcess):
 		if isinstance(retval, int):
 			# spawn failed
 			self._unregister()
-			self.returncode = retval
+			self._set_returncode((self.pid, retval))
 			self.wait()
 			return
 
@@ -143,14 +150,10 @@ class SpawnProcess(SubProcess):
 
 	def _output_handler(self, fd, event):
 
-		if event & PollConstants.POLLIN:
+		files = self._files
+		buf = self._read_buf(files.process, event)
 
-			files = self._files
-			buf = array.array('B')
-			try:
-				buf.fromfile(files.process, self._bufsize)
-			except EOFError:
-				pass
+		if buf is not None:
 
 			if buf:
 				if not self.background:
@@ -189,14 +192,17 @@ class SpawnProcess(SubProcess):
 								fcntl.fcntl(files.stdout.fileno(),
 								fcntl.F_GETFL) ^ os.O_NONBLOCK)
 
-				buf.tofile(files.log)
+				try:
+					buf.tofile(files.log)
+				except TypeError:
+					# array.tofile() doesn't work with GzipFile
+					files.log.write(buf.tostring())
 				files.log.flush()
 			else:
 				self._unregister()
 				self.wait()
 
 		self._unregister_if_appropriate(event)
-		return self._registered
 
 	def _dummy_handler(self, fd, event):
 		"""
@@ -205,13 +211,9 @@ class SpawnProcess(SubProcess):
 		monitor the process from inside a poll() loop.
 		"""
 
-		if event & PollConstants.POLLIN:
+		buf = self._read_buf(self._files.process, event)
 
-			buf = array.array('B')
-			try:
-				buf.fromfile(self._files.process, self._bufsize)
-			except EOFError:
-				pass
+		if buf is not None:
 
 			if buf:
 				pass
@@ -220,5 +222,4 @@ class SpawnProcess(SubProcess):
 				self.wait()
 
 		self._unregister_if_appropriate(event)
-		return self._registered
 
