@@ -1,15 +1,15 @@
-# Copyright 1999-2009 Gentoo Foundation
+# Copyright 1999-2011 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Id: help.py 15687 2010-03-02 21:07:53Z zmedico $
 
 from __future__ import print_function
 
+from portage.const import _ENABLE_DYN_LINK_MAP
 from portage.output import bold, turquoise, green
 
 def shorthelp():
 	print(bold("emerge:")+" the other white meat (command-line interface to the Portage system)")
 	print(bold("Usage:"))
-	print("   "+turquoise("emerge")+" [ "+green("options")+" ] [ "+green("action")+" ] [ "+turquoise("ebuild")+" | "+turquoise("tbz2")+" | "+turquoise("file")+" | "+turquoise("set")+" | "+turquoise("atom")+" ] [ ... ]")
+	print("   "+turquoise("emerge")+" [ "+green("options")+" ] [ "+green("action")+" ] [ "+turquoise("ebuild")+" | "+turquoise("tbz2")+" | "+turquoise("file")+" | "+turquoise("@set")+" | "+turquoise("atom")+" ] [ ... ]")
 	print("   "+turquoise("emerge")+" [ "+green("options")+" ] [ "+green("action")+" ] < "+turquoise("system")+" | "+turquoise("world")+" >")
 	print("   "+turquoise("emerge")+" < "+turquoise("--sync")+" | "+turquoise("--metadata")+" | "+turquoise("--info")+" >")
 	print("   "+turquoise("emerge")+" "+turquoise("--resume")+" [ "+green("--pretend")+" | "+green("--ask")+" | "+green("--skipfirst")+" ]")
@@ -21,7 +21,7 @@ def shorthelp():
 	print("          [ "+green("--newuse")+"    ] [ "+green("--noconfmem")+"  ] [ "+green("--nospinner")+"  ]")
 	print("          [ "+green("--oneshot")+"   ] [ "+green("--onlydeps")+"   ]")
 	print("          [ "+green("--reinstall ")+turquoise("changed-use")+"      ] [ " + green("--with-bdeps")+" < " + turquoise("y") + " | "+ turquoise("n")+" >         ]")
-	print(bold("Actions:")+"  [ "+green("--depclean")+" | "+" | "+green("--search")+" | "+green("--sync")+" | "+green("--version")+"        ]")
+	print(bold("Actions:")+"  [ "+green("--depclean")+" | "+green("--list-sets")+" | "+green("--search")+" | "+green("--sync")+" | "+green("--version")+"        ]")
 
 def help(myopts, havecolor=1):
 	# TODO: Implement a wrap() that accounts for console color escape codes.
@@ -70,8 +70,8 @@ def help(myopts, havecolor=1):
 
 		paragraph = "Cleans the system by removing packages that are " + \
 		"not associated with explicitly merged packages. Depclean works " + \
-		"by creating the full dependency tree from the system and " + \
-		"world sets, then comparing it to installed packages. Packages " + \
+		"by creating the full dependency tree from the " + \
+		"@world set, then comparing it to installed packages. Packages " + \
 		"installed, but not part of the dependency tree, will be " + \
 		"uninstalled by depclean. See --with-bdeps for behavior with " + \
 		"respect to build time dependencies that are not strictly " + \
@@ -80,7 +80,7 @@ def help(myopts, havecolor=1):
 		"emerge --noreplace <atom>. As a safety measure, depclean " + \
 		"will not remove any packages unless *all* required dependencies " + \
 		"have been resolved. As a consequence, it is often necessary to " + \
-		"run emerge --update --newuse --deep world " + \
+		"run emerge --update --newuse --deep @world " + \
 		"prior to depclean."
 
 		for line in wrap(paragraph, desc_width):
@@ -92,10 +92,18 @@ def help(myopts, havecolor=1):
 		"packages will be uninstalled. Always study the list of packages " + \
 		"to be cleaned for any obvious mistakes. Note that packages " + \
 		"listed in package.provided (see portage(5)) may be removed by " + \
-		"depclean, even if they are part of the world set. Also note that " + \
-		"depclean may break link level dependencies. Thus, it is " + \
-		"recommended to use a tool such as revdep-rebuild(1) " + \
-		"in order to detect such breakage."
+		"depclean, even if they are part of the world set."
+
+		paragraph += " Also note that " + \
+			"depclean may break link level dependencies"
+
+		if _ENABLE_DYN_LINK_MAP:
+			paragraph += ", especially when the " + \
+				"--depclean-lib-check option is disabled"
+
+		paragraph += ". Thus, it is " + \
+			"recommended to use a tool such as revdep-rebuild(1) " + \
+			"in order to detect such breakage."
 
 		for line in wrap(paragraph, desc_width):
 			print(desc_indent + line)
@@ -109,15 +117,24 @@ def help(myopts, havecolor=1):
 		for line in wrap(paragraph, desc_width):
 			print(desc_indent + line)
 		print()
-		print("       " + green("--deselect") + "[=%s]" % turquoise("n"))
+		print("       " + green("--deselect") + " [ %s | %s ]" % \
+			(turquoise("y"), turquoise("n")))
 
 		paragraph = \
-			"Remove atoms from the world file. This action is implied " + \
+			"Remove atoms and/or sets from the world file. This action is implied " + \
 			"by uninstall actions, including --depclean, " + \
 			"--prune and --unmerge. Use --deselect=n " + \
 			"in order to prevent uninstall actions from removing " + \
 			"atoms from the world file."
 
+		for line in wrap(paragraph, desc_width):
+			print(desc_indent + line)
+		print()
+		print("       " + green("--ignore-default-opts"))
+
+		paragraph = \
+			"Causes EMERGE_DEFAULT_OPTS (see make.conf(5)) to be ignored."
+
 		for line in wrap(paragraph, desc_width):
 			print(desc_indent + line)
 		print()
@@ -128,6 +145,12 @@ def help(myopts, havecolor=1):
 		print("              make.{conf,globals,defaults} and the environment show up if")
 		print("              run with the '--verbose' flag.")
 		print()
+		print("       " + green("--list-sets"))
+		paragraph = "Displays a list of available package sets."
+
+		for line in wrap(paragraph, desc_width):
+			print(desc_indent + line)
+		print()
 		print("       "+green("--metadata"))
 		print("              Transfers metadata cache from ${PORTDIR}/metadata/cache/ to")
 		print("              /var/cache/edb/dep/ as is normally done on the tail end of an")
@@ -195,18 +218,31 @@ def help(myopts, havecolor=1):
 		print("                emerge -S 'perl.*module'")
 		print()
 		print("       "+green("--sync"))
-		print("              Tells emerge to update the Portage tree as specified in")
-		print("              The SYNC variable found in /etc/make.conf.  By default, SYNC instructs")
-		print("              emerge to perform an rsync-style update with rsync.gentoo.org.")
+		desc = "This updates the portage tree that is located in the " + \
+			"directory that the PORTDIR variable refers to (default " + \
+			"location is /usr/portage). The SYNC variable specifies " + \
+			"the remote URI from which files will be synchronized. " + \
+			"The PORTAGE_SYNC_STALE variable configures " + \
+			"warnings that are shown when emerge --sync has not " + \
+			"been executed recently."
+		for line in wrap(desc, desc_width):
+			print(desc_indent + line)
 		print()
-		print("              'emerge-webrsync' exists as a helper app to emerge --sync, providing a")
-		print("              method to receive the entire portage tree as a tarball that can be")
-		print("              extracted and used. First time syncs would benefit greatly from this.")
+		print(desc_indent + turquoise("WARNING:"))
+		desc = "The emerge --sync action will modify and/or delete " + \
+			"files located inside the directory that the PORTDIR " + \
+			"variable refers to (default location is /usr/portage). " + \
+			"For more information, see the PORTDIR documentation in " + \
+			"the make.conf(5) man page."
+		for line in wrap(desc, desc_width):
+			print(desc_indent + line)
 		print()
-		print("              "+turquoise("WARNING:"))
-		print("              If using our rsync server, emerge will clean out all files that do not")
-		print("              exist on it, including ones that you may have created. The exceptions")
-		print("              to this are the distfiles, local and packages directories.")
+		print(desc_indent + green("NOTE:"))
+		desc = "The emerge-webrsync program will download the entire " + \
+			"portage tree as a tarball, which is much faster than emerge " + \
+			"--sync for first time syncs."
+		for line in wrap(desc, desc_width):
+			print(desc_indent + line)
 		print()
 		print("       "+green("--unmerge")+" ("+green("-C")+" short option)")
 		print("              "+turquoise("WARNING: This action can remove important packages!"))
@@ -216,13 +252,6 @@ def help(myopts, havecolor=1):
 		print("              ebuilds. For a dependency aware version of --unmerge, use")
 		print("              --depclean or --prune.")
 		print()
-		print("       "+green("--update")+" ("+green("-u")+" short option)")
-		print("              Updates packages to the best version available, which may not")
-		print("              always be the highest version number due to masking for testing")
-		print("              and development. Package atoms specified on the command")
-		print("              line are greedy, meaning that unspecific atoms may match multiple")
-		print("              installed versions of slotted packages.")
-		print()
 		print("       "+green("--version")+" ("+green("-V")+" short option)")
 		print("              Displays the currently installed version of portage along with")
 		print("              other information useful for quick reference on a system. See")
@@ -275,22 +304,37 @@ def help(myopts, havecolor=1):
 			"EMERGE_DEFAULT_OPTS variable."
 		for line in wrap(desc, desc_width):
 			print(desc_indent + line)
+		print() 
+		print("       " + green("--autounmask") + " [ %s | %s ]" % \
+			(turquoise("y"), turquoise("n")))
+		desc = "Automatically unmask packages. If any configuration " + \
+			"changes are required, then they will be displayed " + \
+			"after the merge list and emerge will immediately " + \
+			"abort. If the displayed configuration changes are " + \
+			"satisfactory, you should copy and paste them into " + \
+			"the specified configuration file(s). Currently, " + \
+			"this only works for unstable KEYWORDS masks, " + \
+			"LICENSE masks, and package.use settings."
+		for line in wrap(desc, desc_width):
+			print(desc_indent + line)
 		print()
 		print("       " + green("--backtrack") + " " + turquoise("COUNT"))
 		desc = "Specifies an integer number of times to backtrack if " + \
 			"dependency calculation fails due to a conflict or an " + \
-			"unsatisfied dependency (default: '5')."
+			"unsatisfied dependency (default: '10')."
 		for line in wrap(desc, desc_width):
 			print(desc_indent + line)
 		print()
-		print("        " + green("--binpkg-respect-use") + \
-			" < " + turquoise("y") + " | " + turquoise("n") + " >")
+		print("        " + green("--binpkg-respect-use") + " [ %s | %s ]" % \
+			(turquoise("y"), turquoise("n")))
 		desc = "Tells emerge to ignore binary packages if their use flags" + \
 			" don't match the current configuration. (default: 'n')"
 		for line in wrap(desc, desc_width):
 			print(desc_indent + line)
 		print()
-		print("       "+green("--buildpkg")+" ("+green("-b")+" short option)")
+		print("       " + green("--buildpkg") + \
+			" [ %s | %s ] (%s short option)" % \
+			(turquoise("y"), turquoise("n"), green("-b")))
 		desc = "Tells emerge to build binary packages for all ebuilds processed in" + \
 			" addition to actually merging the packages. Useful for maintainers" + \
 			" or if you administrate multiple Gentoo Linux systems (build once," + \
@@ -327,9 +371,10 @@ def help(myopts, havecolor=1):
 		print("              Display the pretend output in a tabular form. Versions are")
 		print("              aligned vertically.")
 		print()
-		print("       "+green("--complete-graph") + "[=%s]" % turquoise("n"))
+		print("       "+green("--complete-graph") + " [ %s | %s ]" % \
+			(turquoise("y"), turquoise("n")))
 		desc = "This causes emerge to consider the deep dependencies of all" + \
-			" packages from the system and world sets. With this option enabled," + \
+			" packages from the world set. With this option enabled," + \
 			" emerge will bail out if it determines that the given operation will" + \
 			" break any dependencies of the packages that have been added to the" + \
 			" graph. Like the --deep option, the --complete-graph" + \
@@ -361,14 +406,36 @@ def help(myopts, havecolor=1):
 		print("              that are not directly listed in the dependencies of a package.")
 		print("              Also see --with-bdeps for behavior with respect to build time")
 		print("              dependencies that are not strictly required.")
-		print() 
-		print("       "+green("--emptytree")+" ("+green("-e")+" short option)")
-		print("              Virtually tweaks the tree of installed packages to contain")
-		print("              nothing. This is great to use together with --pretend. This makes")
-		print("              it possible for developers to get a complete overview of the")
-		print("              complete dependency tree of a certain package.")
 		print()
-		print("       "+green("--fail-clean[=n]"))
+
+		if _ENABLE_DYN_LINK_MAP:
+			print("       " + green("--depclean-lib-check") + " [ %s | %s ]" % \
+				(turquoise("y"), turquoise("n")))
+			desc = "Account for library link-level dependencies during " + \
+				"--depclean and --prune actions. This " + \
+				"option is enabled by default. In some cases this can " + \
+				"be somewhat time-consuming."
+			for line in wrap(desc, desc_width):
+				print(desc_indent + line)
+			print()
+
+		print("       "+green("--emptytree")+" ("+green("-e")+" short option)")
+		desc = "Reinstalls target atoms and their entire deep " + \
+			"dependency tree, as though no packages are currently " + \
+			"installed. You should run this with --pretend " + \
+			"first to make sure the result is what you expect."
+		for line in wrap(desc, desc_width):
+			print(desc_indent + line)
+		print()
+		print("       " + green("--exclude") + " " + turquoise("ATOMS"))
+		desc = "A space separated list of package names or slot atoms. " + \
+			"Emerge won't  install any ebuild or binary package that " + \
+			"matches any of the given package atoms."
+		for line in wrap(desc, desc_width):
+			print(desc_indent + line)
+		print()
+		print("       " + green("--fail-clean") + " [ %s | %s ]" % \
+			(turquoise("y"), turquoise("n")))
 		desc = "Clean up temporary files after a build failure. This is " + \
 			"particularly useful if you have PORTAGE_TMPDIR on " + \
 			"tmpfs. If this option is enabled, you probably also want " + \
@@ -387,13 +454,17 @@ def help(myopts, havecolor=1):
 		print("              Same as --fetchonly except that all package files, including those")
 		print("              not required to build the package, will be processed.")
 		print()
-		print("       "+green("--getbinpkg")+ "[=%s]" % turquoise("n") + " ("+green("-g")+" short option)")
+		print("       " + green("--getbinpkg") + \
+			" [ %s | %s ] (%s short option)" % \
+			(turquoise("y"), turquoise("n"), green("-g")))
 		print("              Using the server and location defined in PORTAGE_BINHOST, portage")
 		print("              will download the information from each binary file there and it")
 		print("              will use that information to help build the dependency list. This")
 		print("              option implies '-k'. (Use -gK for binary-only merging.)")
 		print()
-		print("       "+green("--getbinpkgonly")+ "[=%s]" % turquoise("n") + " ("+green("-G")+" short option)")
+		print("       " + green("--getbinpkgonly") + \
+			" [ %s | %s ] (%s short option)" % \
+			(turquoise("y"), turquoise("n"), green("-G")))
 		print("              This option is identical to -g, as above, except it will not use")
 		print("              ANY information from the local machine. All binaries will be")
 		print("              downloaded from the remote server without consulting packages")
@@ -412,7 +483,8 @@ def help(myopts, havecolor=1):
 		for line in wrap(desc, desc_width):
 			print(desc_indent + line)
 		print()
-		print("       " + green("--keep-going") + "[=%s]" % turquoise("n"))
+		print("       " + green("--keep-going") + " [ %s | %s ]" % \
+			(turquoise("y"), turquoise("n")))
 		desc = "Continue as much as possible after " + \
 			"an error. When an error occurs, " + \
 			"dependencies are recalculated for " + \
@@ -441,7 +513,10 @@ def help(myopts, havecolor=1):
 		print("       "+green("--newuse")+" ("+green("-N")+" short option)")
 		desc = "Tells emerge to include installed packages where USE " + \
 			"flags have changed since compilation. This option " + \
-			"also implies the --selective option."
+			"also implies the --selective option. If you would " + \
+			"like to skip rebuilds for which disabled flags have " + \
+			"been added to or removed from IUSE, see the related " + \
+			"--reinstall=changed-use option."
 		for line in wrap(desc, desc_width):
 			print(desc_indent + line)
 		print()
@@ -482,6 +557,16 @@ def help(myopts, havecolor=1):
 		print("              Only merge (or pretend to merge) the dependencies of the")
 		print("              specified packages, not the packages themselves.")
 		print()
+		print("       " + green("--package-moves") + " [ %s | %s ]" % \
+			(turquoise("y"), turquoise("n")))
+		desc = "Perform package moves when necessary. This option " + \
+			"is enabled by default. WARNING: This option " + \
+			"should remain enabled under normal circumstances. " + \
+			"Do not disable it unless you know what you are " + \
+			"doing."
+		for line in wrap(desc, desc_width):
+			print(desc_indent + line)
+		print()
 		print("       "+green("--pretend")+" ("+green("-p")+" short option)")
 		print("              Instead of actually performing the merge, simply display what")
 		print("              ebuilds and tbz2s *would* have been installed if --pretend")
@@ -511,7 +596,8 @@ def help(myopts, havecolor=1):
 		for line in wrap(desc, desc_width):
 			print(desc_indent + line)
 		print()
-		print("       "+green("--rebuilt-binaries") + "[=%s]" % turquoise("n"))
+		print("       " + green("--rebuilt-binaries") + " [ %s | %s ]" % \
+			(turquoise("y"), turquoise("n")))
 		desc = "Replace installed packages with binary packages that have " + \
 			"been rebuilt. Rebuilds are detected by comparison of " + \
 			"BUILD_TIME package metadata. This option is enabled " + \
@@ -521,6 +607,15 @@ def help(myopts, havecolor=1):
 		for line in wrap(desc, desc_width):
 			print(desc_indent + line)
 		print()
+		print("       "+green("--rebuilt-binaries-timestamp") + "=%s" % turquoise("TIMESTAMP"))
+		desc = "This option modifies emerge's behaviour only if " + \
+			"--rebuilt-binaries is given. Only binaries that " + \
+			"have a BUILD_TIME that is larger than the given TIMESTAMP " + \
+			"and that is larger than that of the installed package will " + \
+			"be considered by the rebuilt-binaries logic."
+		for line in wrap(desc, desc_width):
+			print(desc_indent + line)
+		print()
 		print("       "+green("--reinstall ") + turquoise("changed-use"))
 		print("              Tells emerge to include installed packages where USE flags have")
 		print("              changed since installation.  Unlike --newuse, this option does")
@@ -546,7 +641,8 @@ def help(myopts, havecolor=1):
 		for line in wrap(desc, desc_width):
 			print(desc_indent + line)
 		print()
-		print("       " + green("--select") + "[=%s]" % turquoise("n"))
+		print("       " + green("--select") + " [ %s | %s ]" % \
+			(turquoise("y"), turquoise("n")))
 		desc = "Add specified packages to the world set (inverse of " + \
 			"--oneshot). This is useful if you want to " + \
 			"use EMERGE_DEFAULT_OPTS to make " + \
@@ -554,7 +650,8 @@ def help(myopts, havecolor=1):
 		for line in wrap(desc, desc_width):
 			print(desc_indent + line)
 		print()
-		print("       " + green("--selective") + "[=%s]" % turquoise("n"))
+		print("       " + green("--selective") + " [ %s | %s ]" % \
+			(turquoise("y"), turquoise("n")))
 		desc = "This is similar to the --noreplace option, except that it " + \
 			"does not take precedence over options such as --newuse. " + \
 			"Some options, such as --update, imply --selective. " + \
@@ -591,13 +688,25 @@ def help(myopts, havecolor=1):
 		for line in wrap(desc, desc_width):
 			print(desc_indent + line)
 		print()
-		print("       " + green("--use-ebuild-visibility") + "[=%s]" % turquoise("n"))
+		print("       "+green("--update")+" ("+green("-u")+" short option)")
+		desc = "Updates packages to the best version available, which may " + \
+			"not always be the  highest version number due to masking " + \
+			"for testing and development. Package atoms specified on " + \
+			"the command line are greedy, meaning that unspecific " + \
+			"atoms may match multiple versions of slotted packages."
+		for line in wrap(desc, desc_width):
+			print(desc_indent + line)
+		print()
+		print("       " + green("--use-ebuild-visibility") + " [ %s | %s ]" % \
+			(turquoise("y"), turquoise("n")))
 		desc = "Use unbuilt ebuild metadata for visibility " + \
 			"checks on built packages."
 		for line in wrap(desc, desc_width):
 			print(desc_indent + line)
 		print()
-		print("       "+green("--usepkg")+ "[=%s]" % turquoise("n") + " ("+green("-k")+" short option)")
+		print("       " + green("--usepkg") + \
+			" [ %s | %s ] (%s short option)" % \
+			(turquoise("y"), turquoise("n"), green("-k")))
 		print("              Tell emerge to use binary packages (from $PKGDIR) if they are")
 		print("              available, thus possibly avoiding some time-consuming compiles.")
 		print("              This option is useful for CD installs; you can export")
@@ -605,7 +714,9 @@ def help(myopts, havecolor=1):
 		print("              emerge \"pull\" binary packages from the CD in order to satisfy") 
 		print("              dependencies.")
 		print()
-		print("       "+green("--usepkgonly")+ "[=%s]" % turquoise("n") + " ("+green("-K")+" short option)")
+		print("       " + green("--usepkgonly") + \
+			" [ %s | %s ] (%s short option)" % \
+			(turquoise("y"), turquoise("n"), green("-K")))
 		print("              Like --usepkg above, except this only allows the use of binary")
 		print("              packages, and it will abort the emerge if the package is not")
 		print("              available at the time of dependency calculation.")
