@@ -1,6 +1,5 @@
-# Copyright 1999-2009 Gentoo Foundation
+# Copyright 1999-2011 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Id: unmerge.py 15611 2010-03-02 20:37:38Z zmedico $
 
 from __future__ import print_function
 
@@ -41,14 +40,14 @@ def unmerge(root_config, myopts, unmerge_action,
 	def _pkg(cpv):
 		pkg = pkg_cache.get(cpv)
 		if pkg is None:
-			pkg = Package(cpv=cpv, installed=True,
+			pkg = Package(built=True, cpv=cpv, installed=True,
 				metadata=zip(db_keys, vartree.dbapi.aux_get(cpv, db_keys)),
-				root_config=root_config,
+				operation="uninstall", root_config=root_config,
 				type_name="installed")
 			pkg_cache[cpv] = pkg
 		return pkg
 
-	vdb_path = os.path.join(settings["ROOT"], portage.VDB_PATH)
+	vdb_path = os.path.join(settings["EROOT"], portage.VDB_PATH)
 	try:
 		# At least the parent needs to exist for the lock file.
 		portage.util.ensure_dirs(vdb_path)
@@ -119,7 +118,7 @@ def unmerge(root_config, myopts, unmerge_action,
 	
 					sp_absx_len = len(sp_absx)
 	
-					vdb_path = os.path.join(settings["ROOT"], portage.VDB_PATH)
+					vdb_path = os.path.join(settings["EROOT"], portage.VDB_PATH)
 					vdb_len  = len(vdb_path)
 	
 					sp_vdb     = vdb_path.split("/")
@@ -187,7 +186,7 @@ def unmerge(root_config, myopts, unmerge_action,
 				mymatch = localtree.dep_match(x)
 			if not mymatch:
 				portage.writemsg("\n--- Couldn't find '%s' to %s.\n" % \
-					(x, unmerge_action), noiselevel=-1)
+					(x.replace("null/", ""), unmerge_action), noiselevel=-1)
 				continue
 
 			pkgmap.append(
@@ -281,7 +280,31 @@ def unmerge(root_config, myopts, unmerge_action,
 		if vdb_lock:
 			vartree.dbapi.flush_cache()
 			portage.locks.unlockdir(vdb_lock)
+	
+	from portage._sets.base import EditablePackageSet
+	
+	# generate a list of package sets that are directly or indirectly listed in "selected",
+	# as there is no persistent list of "installed" sets
+	installed_sets = ["selected"]
+	stop = False
+	pos = 0
+	while not stop:
+		stop = True
+		pos = len(installed_sets)
+		for s in installed_sets[pos - 1:]:
+			if s not in sets:
+				continue
+			candidates = [x[len(SETPREFIX):] for x in sets[s].getNonAtoms() if x.startswith(SETPREFIX)]
+			if candidates:
+				stop = False
+				installed_sets += candidates
+	installed_sets = [x for x in installed_sets if x not in root_config.setconfig.active]
+	del stop, pos
 
+	# we don't want to unmerge packages that are still listed in user-editable package sets
+	# listed in "world" as they would be remerged on the next update of "world" or the 
+	# relevant package sets.
+	unknown_sets = set()
 	for cp in range(len(pkgmap)):
 		for cpv in pkgmap[cp]["selected"].copy():
 			try:
@@ -305,6 +328,61 @@ def unmerge(root_config, myopts, unmerge_action,
 				pkgmap[cp]["protected"].add(cpv)
 				continue
 
+			parents = []
+			for s in installed_sets:
+				# skip sets that the user requested to unmerge, and skip world 
+				# user-selected set, since the package will be removed from
+				# that set later on.
+				if s in root_config.setconfig.active or s == "selected":
+					continue
+
+				if s not in sets:
+					if s in unknown_sets:
+						continue
+					unknown_sets.add(s)
+					out = portage.output.EOutput()
+					out.eerror(("Unknown set '@%s' in %s%s") % \
+						(s, root_config.settings['EROOT'], portage.const.WORLD_SETS_FILE))
+					continue
+
+				# only check instances of EditablePackageSet as other classes are generally used for
+				# special purposes and can be ignored here (and are usually generated dynamically, so the
+				# user can't do much about them anyway)
+				if isinstance(sets[s], EditablePackageSet):
+
+					# This is derived from a snippet of code in the
+					# depgraph._iter_atoms_for_pkg() method.
+					for atom in sets[s].iterAtomsForPackage(pkg):
+						inst_matches = vartree.dbapi.match(atom)
+						inst_matches.reverse() # descending order
+						higher_slot = None
+						for inst_cpv in inst_matches:
+							try:
+								inst_pkg = _pkg(inst_cpv)
+							except KeyError:
+								# It could have been uninstalled
+								# by a concurrent process.
+								continue
+
+							if inst_pkg.cp != atom.cp:
+								continue
+							if pkg >= inst_pkg:
+								# This is descending order, and we're not
+								# interested in any versions <= pkg given.
+								break
+							if pkg.slot_atom != inst_pkg.slot_atom:
+								higher_slot = inst_pkg
+								break
+						if higher_slot is None:
+							parents.append(s)
+							break
+			if parents:
+				print(colorize("WARN", "Package %s is going to be unmerged," % cpv))
+				print(colorize("WARN", "but still listed in the following package sets:"))
+				print("    %s\n" % ", ".join(parents))
+
+	del installed_sets
+
 	numselected = len(all_selected)
 	if not numselected:
 		writemsg_level(
@@ -349,10 +427,10 @@ def unmerge(root_config, myopts, unmerge_action,
 			#avoid cluttering the preview printout with stuff that isn't getting unmerged
 			continue
 		if not (pkgmap[x]["protected"] or pkgmap[x]["omitted"]) and cp in syslist:
-			writemsg_level(colorize("BAD","\a\n\n!!! " + \
+			writemsg_level(colorize("BAD","\n\n!!! " + \
 				"'%s' is part of your system profile.\n" % cp),
 				level=logging.WARNING, noiselevel=-1)
-			writemsg_level(colorize("WARN","\a!!! Unmerging it may " + \
+			writemsg_level(colorize("WARN","!!! Unmerging it may " + \
 				"be damaging to your system.\n\n"),
 				level=logging.WARNING, noiselevel=-1)
 			if clean_delay and "--pretend" not in myopts and "--ask" not in myopts:
@@ -387,6 +465,8 @@ def unmerge(root_config, myopts, unmerge_action,
 		if quiet:
 			writemsg_level("\n", noiselevel=-1)
 
+	writemsg_level("\nAll selected packages: %s\n" % " ".join(all_selected), noiselevel=-1)
+
 	writemsg_level("\n>>> " + colorize("UNMERGE_WARN", "'Selected'") + \
 		" packages are slated for removal.\n")
 	writemsg_level(">>> " + colorize("GOOD", "'Protected'") + \
